<?php
// $Horde: horde/lib/Auth/sql.php,v 1.21.2.3 2002/04/05 01:24:26 jon Exp $

/**
 * The Auth_sql class provides a sql implementation of the Horde
 * authentication system.
 *
 * Required values for $params:
 *      'phptype'       The database type (ie. 'pgsql', 'mysql, etc.).
 *      'hostspec'      The hostname of the database server.
 *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'database'      The name of the database.
 *      'table'         The name of the auth table in 'database'.
 *
 * Required by some database implementations:
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.
 *
 * The table structure for the auth system is as follows:
 *
 *  create table horde_users (
 *      user_uid        varchar(255) not null,
 *      user_pass       varchar(255) not null,
 *      primary key (user_uid)
 *  );
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.21.2.3 $
 * @since   Horde 1.3
 * @package horde.auth
 */
class Auth_sql extends Auth {

    /** An array of capabilities, so that the driver can report which
        operations it supports and which it doesn't.
        @var array $capabilities */
    var $capabilities = array('add'         => true,
                              'update'      => true,
                              'remove'      => true,
                              'list'        => true,
                              'transparent' => false,
                              'loginscreen' => false);

    /**
     * Hash containing connection parameters.
     * @var array $params
     */
    var $params = array();

    /**
     * Handle for the current database connection.
     * @var object DB $db
     */
    var $db;

    /**
     * Boolean indicating whether or not we're connected to the SQL server.
     * @var boolean $connected
     */
    var $connected = false;


    /**
     * Constructs a new SQL authentication object.
     *
     * @param array $params   A hash containing connection parameters.
     */
    function Auth_sql($params = array())
    {
        $this->params = $params;
    }


    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return mixed True on success or a PEAR_Error object on failure.
     */
    function _connect()
    {
        if (!$this->connected) {
            if (!is_array($this->params)) {
                Horde::fatal(new PEAR_Error(_("No configuration information specified for SQL authentication.")), __FILE__, __LINE__);
            }
            if (!isset($this->params['phptype'])) {
                Horde::fatal(new PEAR_Error(_("Required 'phptype' not specified in authentication configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->params['hostspec'])) {
                Horde::fatal(new PEAR_Error(_("Required 'hostspec' not specified in authentication configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->params['username'])) {
                Horde::fatal(new PEAR_Error(_("Required 'username' not specified in authentication configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->params['password'])) {
                Horde::fatal(new PEAR_Error(_("Required 'password' not specified in authentication configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->params['database'])) {
                Horde::fatal(new PEAR_Error(_("Required 'database' not specified in authentication configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->params['table'])) {
                Horde::fatal(new PEAR_Error(_("Required 'table' not specified in authentication configuration.")), __FILE__, __LINE__);
            }

            /* Connect to the SQL server using the supplied parameters. */
            include_once 'DB.php';
            $this->db = &DB::connect($this->params, true);
            if (DB::isError($this->db) || DB::isWarning($this->db)) {
                Horde::fatal(new PEAR_Error(_("Unable to connect to SQL server.")), __FILE__, __LINE__);
            }

            /* Enable the "portability" option. */
            $this->db->setOption('optimize', 'portability');

            $this->connected = true;
        }

        return true;
    }

    /**
     * Disconnect from the SQL server and clean up the connection.
     *
     * @return boolean true on success, false on failure.
     */
    function _disconnect()
    {
        if ($this->connected) {
            $this->connected = false;
            return $this->db->disconnect();
        }

        return true;
    }

    /**
     * Find out if a set of login credentials are valid.
     *
     * @param string $userID       The userID to check.
     * @param array  $credentials  The credentials to use.
     *
     * @return boolean Whether or not the credentials are valid.
     */
    function authenticate($userID, $credentials)
    {
        if (Auth::checkAuth($userID)) {
            return true;
        }

        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect();

        /* Build the SQL query. */
        $query = 'SELECT user_uid FROM ' . $this->params['table'];
        $query .= ' WHERE user_uid = ' . $this->db->quote($userID);
        $query .= ' AND user_pass = ' . $this->db->quote(md5($credentials['password']));

        /* Execute the query. */
        $result = $this->db->query($query);

        if (!DB::isError($result)) {
            $row = $result->fetchRow(DB_GETMODE_ASSOC);
            if (is_array($row)) {
                $result->free();
                Auth::setAuth($userID, $credentials);
                return true;
            } else {
                $result->free();
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Add a set of authentication credentials.
     *
     * @param string $userID       The userID to add.
     * @param array  $credentials  The credentials to add.
     *
     * @return mixed        True on success or a PEAR_Error object on failure.
     */
    function addUser($userID, $credentials)
    {
        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect();

        /* Build the SQL query. */
        $query = 'INSERT INTO ' . $this->params['table'] . ' (user_uid, user_pass) ';
        $query .= 'VALUES (' . $this->db->quote($userID) . ', ' . $this->db->quote(md5($credentials['password'])) . ')';

        /* Execute the query. */
        $result = $this->db->query($query);

        if ($result !== DB_OK) {
            return (new PEAR_Error(_("Database query failed.")));
        }

        return true;
    }

    /**
     * Update a set of authentication credentials.
     *
     * @param string $oldID        The old userID.
     * @param string $newID        The new userID.
     * @param array  $credentials  The new credentials
     *
     * @return mixed        True on success or a PEAR_Error object on failure.
     */
    function updateUser($oldID, $newID, $credentials)
    {
        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect();

        /* Build the SQL query. */
        $query = 'UPDATE ' . $this->params['table'] . ' SET user_uid = ' . $this->db->quote($newID) . ', user_pass = ' . $this->db->quote(md5($credentials['password']));
        $query .= 'WHERE user_uid = ' . $this->db->quote($oldID);

        /* Execute the query. */
        $result = $this->db->query($query);

        if ($result !== DB_OK) {
            return (new PEAR_Error(_("Database query failed.")));
        }

        return true;
    }

    /**
     * Delete a set of authentication credentials.
     *
     * @param string $userID  The userID to delete.
     * @return boolean        Success or failure.
     */
    function removeUser($userID)
    {
        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect();

        /* Build the SQL query. */
        $query = 'DELETE FROM ' . $this->params['table'];
        $query .= ' WHERE user_uid = ' . $this->db->quote($userID);

        /* Execute the query. */
        $result = $this->db->query($query);

        if ($result !== DB_OK) {
            return (new PEAR_Error(_("Database query failed.")));
        }

        return true;
    }

    /**
     * List all users in the system.
     *
     * @return mixed   The array of userIDs, or false on failure/unsupported.
     */
    function listUsers()
    {
        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect();

        /* Build the SQL query. */
        $query = 'SELECT user_uid from ' . $this->params['table'];

        /* Execute the query. */
        $result = $this->db->getAll($query, null, DB_FETCHMODE_ORDERED);

        /* Loop through and build return value. */
        $users = array();
        foreach ($result as $ar) {
            $users[] = $ar[0];
        }

        return $users;
    }

}
?>
